"""

File:    PbxConfigurationInfo.py
Author:  Jonathan Maurice <jmaurice@sangoma.com>
Date:    4/7/2011 11:50:06 AM
 
Description: Contain information about the pbx configuration file.
 
Copyright (C) 2003-2011 Sangoma Technologies. 
All Rights Reserved.

This code is Sangoma Technologies Confidential Property.
Use of and Access to this code is covered by a previously executed
Non-disclosure Agreement between Sangoma Technologies and the Recipient.
This code is being supplied for evaluation purposes only and is not to be
used for any other purpose.  

"""

import os
import xml.dom.minidom as minidom

from paraxip.infra.PyLoggingFunctions import *
from netborderPyInfra import TraceScope


fsLogger = Logger("netborder.pbxConfiguration.PbxConfigurationInfo")

class PbxConfigurationInfo(object):
  """This class store attribute information of the root element inside the pbx
  configuration xml file (.pbx)"""

  PBXCFG_ELEM_NAME = 'pbxConfiguration'
  PBXCFG_ATTR_NAME = ['version', 'applicationName', 'applicationVersion',
                      'oemName', 'oemVersion'] 

  def __init__(self, pbxFilePath=None, pbxInfoDict=None):
    t = TraceScope(fsLogger, "PbxConfigurationInfo.__init__")
    super(PbxConfigurationInfo, self).__init__()
    self.configInfoDict = {}

    for attr in self.PBXCFG_ATTR_NAME:
      self.configInfoDict[attr] = None

    if pbxFilePath is not None:
      if not self.setConfigInfoFromFile(pbxFilePath):
        raise Exception("Invalid xml configuration file %s" % pbxFilePath)

    elif pbxInfoDict is not None:
      if not self.setConfigInfo(pbxInfoDict):
        raise Exception("Can't create PbxConfigurationInfo from dictionnary "\
                        "%s" % pbxInfoDict)

    
  def setConfigInfo(self, infoDict):
    """Set internal dictionnary element to element of infoDict."""
    t = TraceScope(fsLogger, "PbxConfigurationInfo.setConfigInfo")
    
    log_debug(fsLogger, "Setting attributes from dictionnary %s" % infoDict)

    for attr in self.configInfoDict.keys():
      if infoDict.has_key(attr):
        log_debug(fsLogger, "Setting attribute %s to %s" %
                 (attr, infoDict[attr])
        )
        self.configInfoDict[attr] = infoDict[attr]
      else:
        log_error(fsLogger, "Can't set pbx configuration info, missing key %s" %
                 attr
        )
        return False

    return True

  def setConfigInfoFromFile(self, pbxFilePath):
    """Set internal dictionnary element by parsing the pbx file"""
    t = TraceScope(fsLogger, "PbxConfigurationInfo.setConfigInfoFromFile")
    
    if not os.path.exists(pbxFilePath):
      log_error(fsLogger, "Pbx configuration file does not exist '%s'" %
                self.pbxFilePath
      )
      return False

    #Parse the pbx file.
    log_debug(fsLogger, "Parsing pbx xml file %s" % pbxFilePath)
    xmlDoc = minidom.parse(pbxFilePath)
    pbxConfigElem = xmlDoc.getElementsByTagName(self.PBXCFG_ELEM_NAME)

    if len(pbxConfigElem) != 1:
      log_error(fsLogger, "Invalid configuration file %s, only one " \
               "pbxConfiguraion element is allowed." % pbxFilePath
      )
      return False

    #Retrieve the pbxConfiguration attributes.
    for elem in pbxConfigElem:
      for attr in self.configInfoDict.keys():

        if elem.hasAttribute(attr):
          attrValue = elem.getAttribute(attr)
          log_debug( fsLogger, "%s attribute %s is %s" % 
            (self.PBXCFG_ELEM_NAME, attr, str(attrValue))
          )
          self.configInfoDict[attr] = str(attrValue)
        else:
          log_error(fsLogger, "Configuration file %s is missing attribute "\
                    "'%s'" % (pbxFilePath, attr)
          )
          return False

    return True


  def getVersion(self):
    """Return version of the application being configured"""
    t = TraceScope(fsLogger, "PbxConfigManager.getAppVersion")
    return self.configInfoDict['version']

  def getAppVersion(self):
    """Return version of the application being configured"""
    t = TraceScope(fsLogger, "PbxConfigManager.getAppVersion")
    return self.configInfoDict['applicationVersion']

  def getAppName(self):
    """Return name of the application beign configured"""
    t = TraceScope(fsLogger, "PbxConfigManager.getAppName")
    return self.configInfoDict['applicationName']

  def getOemName(self):
    """Return oem name of the provided pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getOemName")
    return self.configInfoDict['oemName']

  def getOemVersion(self):
    """Return oem version of the provided pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getOemVersion")
    return self.configInfoDict['oemVersion']


###############################################################################
# Runtest
if __name__ == '__main__':
  
  testFail = False

  fsLogger.setLogLevel(LogLevel.debug)

  pbxFile = os.path.join('test-input', '3cx-a200-2fxo-2fxs.pbx')

  #This is the same information as in the pbx file 3cx-a200-2fxo-2fxs.pbx
  configInfo = {'version':'1.0',
                'applicationName':'Netborder Express Gateway',
                'applicationVersion':'4.1.1',
                'oemName':'3CX',
                'oemVersion':'9.0'}

  pbxCfgInfo = PbxConfigurationInfo(pbxFilePath=pbxFile)

  if pbxCfgInfo is None:
    log_error(fsLogger, "Failed to create PbxConfigurationInfo")

  #Validate that the file was parsed properly.
  if not configInfo['version'] == pbxCfgInfo.getVersion():
    log_error(fsLogger, "Failed to retrieve version '%s' found '%s'" %
             (configInfo['version'], pbxCfgInfo.getVersion())
    )
    testFail = True

  if not configInfo['applicationName'] == pbxCfgInfo.getAppName():
    log_error(fsLogger, "Failed to retrieve application name '%s' found '%s'" %
             (configInfo['applicationName'], pbxCfgInfo.getAppName())
    )
    testFail = True

  if not configInfo['applicationVersion'] == pbxCfgInfo.getAppVersion():
    log_error(fsLogger, "Failed to retrieve version '%s' found '%s'" %
             (configInfo['applicationVersion'], pbxCfgInfo.getAppVersion())
    )
    testFail = True

  if not configInfo['oemName'] == pbxCfgInfo.getOemName():
    log_error(fsLogger, "Failed to retrieve version '%s' found '%s'" %
             (configInfo['oemName'], pbxCfgInfo.getOemName())
    )
    testFail = True

  if not configInfo['oemVersion'] == pbxCfgInfo.getOemVersion():
    log_error(fsLogger, "Failed to retrieve version '%s' found '%s'" %
             (configInfo['oemVersion'], pbxCfgInfo.getOemVersion())
    )
    testFail = True

  if testFail:
    log_error(fsLogger, "Runtest failed!")
    print("Failed")
  else:
    log_info(fsLogger, "Runtest passed!")
    print("Passed")

## !!MIRROR FILE, DO NOT MODIFY!!
