#
#  @file   Converter.py
#  @author Jean-Philippe Boily, jr. eng.
#  @date   02/15/2011
#
#  Description:
#
#  Generate the export.txt file that contains the current application
#  information and creates an archive of the configuration. If the archive name
#  is not specified, date-time_appName_appVersion is used.
#
#  Copyright (C) 2011 Sangoma Technologies.
#  All Rights Reserved.
#
#  This code is Sangoma Technologies Confidential Property.
#  Use of and Access to this code is covered by a previously executed
#  Non-disclosure Agreement between Sangoma Technologies and the Recipient.
#  This code is being supplied for evaluation purposes only and is not to be
#  used for any other purpose.
#
#

__author__="Jean-Philippe Boily, jr. eng."
__date__ ="$15-Feb-2011 4:51:03 PM$"

__version__="1.0"

#-----------------------------------------------------------------------------#
# imports
#-----------------------------------------------------------------------------#
import os
import platform
import re
import netborderPyThread
import subprocess

from zipfile import ZipFile, ZIP_DEFLATED, ZIP_STORED
from datetime import datetime
from paraxip.infra.PyLoggingFunctions import *

#-----------------------------------------------------------------------------#
# global variables
#-----------------------------------------------------------------------------#
fileScopeLogger = Logger("netborder.smartUpdate.ExportCfg")
ConfigExporterLogger = Logger("netborder.smartUpdate.ConfigExporter")


class ConfigExporter(object):
  """Export configuration in a zip file format"""

  def __init__(self, exportCfgPath, outputDir, appExePath,
               appInstDir, exportFileName='', archiveName='', 
               appServiceName=''):

    t = TraceScope(ConfigExporterLogger, "ConfigExporter.__init__")
    super(ConfigExporter, self).__init__()
    self.exportCfgPath = exportCfgPath
    self.outputDir = outputDir
    self.appExePath = appExePath
    self.appInstDir = appInstDir
    self.exportFileName = exportFileName
    self.archiveName = archiveName
    
    self.appBrand = None
    self.appSubBrand = None
    self.appName = None
    self.appVersion = None
    self.appPlatform = None
    self.appServiceName = appServiceName
    self.appSvcAutoStart = None
    self.appSvcRestartOnFail = None


  def setAppInfo(self, strAppBrand=None, strAppSubBrand=None, strAppName=None,
                 strAppVersion=None, strAppPlatform=None, 
                 strAppServiceName=None, bAppSvcAutoStart=None,
                 bAppSvcRestartOnFail=None):
    """Use this method if you want to manually set the application informations.
       Otherwise, the information will be retrieved by invoking the
       application."""
    t = TraceScope(ConfigExporterLogger, "ConfigExporter.configureAppInfo")

    if strAppBrand is not None:
      self.appBrand = strAppBrand

    if strAppSubBrand is not None:
      self.appSubBrand = strAppSubBrand

    if strAppName is not None:
      self.appName = strAppName
      
    if strAppVersion is not None:
      self.appVersion = strAppVersion

    if strAppPlatform is not None:
      self.appPlatform = strAppPlatform

    if strAppServiceName is not None:
      self.appServiceName= strAppServiceName

    if bAppSvcAutoStart is not None:
      self.appSvcAutoStart = bAppSvcAutoStart

    if bAppSvcRestartOnFail is not None:
      self.appSvcRestartOnFail = bAppSvcRestartOnFail

    return True
    

  def setMissingAppInfo(self):
    """Set the app info attributes that are not defined yet."""
    t = TraceScope(ConfigExporterLogger, "ConfigExporter.setMissingAppInfo")

    # Call app cmd to get required information
    log_debug(ConfigExporterLogger, "applicationCmd=%s" % self.appExePath)

    appProcess = subprocess.Popen(args=[self.appExePath,"--version"], 
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)

    stdout, stderr = appProcess.communicate()

    if stderr:
      log_fatal(
        ConfigExporterLogger,
        "failed to validate the application information. error=%s" % stderr
      )
      return False

    if stdout == "": #stdout is a empty string
      log_error(
        ConfigExporterLogger,
        "%s --version returned with no output. The application might be "
        "corrupted." % self.appExePath
      )
      return False

    log_debug(ConfigExporterLogger,"netborder-gateway ouptut: %s" % stdout)
    appInfoLines = stdout.split('\r\n')
    appInfo = appInfoLines[0].split()

    # At this point, appInfo is an array containing the following information
    # appInfo[0] = 'INFO'
    # appInfo[1] = '-'
    # appInfo[2] = '@brand'
    # appInfo[3] = '@sub_brand'
    # appInfo[4] = '@appType'
    # appInfo[5] = 'version'
    # appInfo[6] = '@versionNum'

    if self.appBrand is None:
      self.appBrand = appInfo[2]

    if self.appSubBrand is None:
      self.appSubBrand = appInfo[3]

    if self.appName is None:
      self.appName = appInfo[4]

    if self.appVersion is None:
      self.appVersion = appInfo[6]

    if self.appPlatform is None:
      self.appPlatform = platform.system()

    return self.setMissingServiceInfo()


  def setMissingServiceInfo(self):
    """Set the service related info that are not defined yet """
    t = TraceScope(ConfigExporterLogger, "ExportCfg.findServiceInfo")

    if self.appServiceName == '':
      log_debug(ConfigExporterLogger, "Retrieving application '%s' product "\
                "name." % self.appExePath)
      appProcess = subprocess.Popen(args=[self.appExePath,"--product-name"],
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)

      stdout, stderr = appProcess.communicate()

      if stderr:
        log_error(
          ConfigExporterLogger,
          "Failed to retrieve application product name. error=%s" % stderr
        )
        return False

      #Retrieve product name
      if platform.system() == "Linux":
        self.appServiceName = self.appName
      else:
        #Remove leading and trailing whitespace.
        for line in stdout.split('\n'):
          m = re.search('Product name : (.+)' , line)
          if m:
            #Remove leading and trailing whitespace.
            self.appServiceName = m.group(1).strip()

      #This shouldn't happen, but if it's the case continue anyway.
      if self.appServiceName == '':
        log_warn(ConfigExporterLogger, "Failed to retrieve application "\
                                  "service name. Service information will "\
                                  "not be exported.")
      else:
        log_debug(ConfigExporterLogger, "Found application product name '%s'" %
                                    self.appServiceName)

    #Retrieve service configuration.
    self.appSvcAutoStart = False
    self.appSvcRestartOnFail = False
    if self.appServiceName != '':
      log_debug(ConfigExporterLogger, "Using service name '%s'" %
                self.appServiceName)
      svcAutoStart = \
          netborderPyThread.isServiceStartTypeAutomatic(self.appServiceName) 
      svcRestartOnFail = \
          netborderPyThread.isServiceFailureAutomaticRestartSet(self.appServiceName)

      if svcAutoStart:
        log_debug(ConfigExporterLogger, "Service '%s' is configured to "\
                  "start automatically on start up" % self.appServiceName)
        self.appSvcAutoStart = True

      if svcRestartOnFail:
        log_debug(ConfigExporterLogger, "Service '%s' is configured to "\
                  "restart automatically on failure" % self.appServiceName)
        self.appSvcRestartOnFail = True
    
    return True


  def export(self):
    """Export the configuration to self.outputDir"""
    t = TraceScope(ConfigExporterLogger, "ConfigExporter.export")

    #Make sure that all application info attributes are defined.
    if not self.setMissingAppInfo():
      log_error(
        ConfigExporterLogger,
        "Could not get application information. Aborting export process."
      )
      return False

    # Set the archive name
    if self.archiveName == '':
      # If the archive name is not specified, the archive name will have the
      # format currentDate-currentTime_appliationName_applicationVersion.zip
      time = datetime.now()
      self.archiveName = str(time.year) + "-" + str(time.month) + "-" + \
                         str(time.day) + "-" + str(time.hour) + "h" + \
                         str(time.minute) + "m" + str(time.second) + "s_" + \
                         self.appBrand + self.appSubBrand + self.appName + \
                         "_" + self.appVersion + ".zip"
    else:

      #Check wether archive name was entered with .zip extension, if not add it.
      archiveNameNoExt, archiveNameExt = os.path.splitext(self.archiveName)
      if archiveNameExt != '.zip':
        self.archiveName += '.zip' 

    log_debug(ConfigExporterLogger,"archive name: %s" % self.archiveName)

    strAppBrand = self.appBrand.lower()
    strAppSubBrand = self.appSubBrand.lower()
    strAppVersion = self.appVersion.lower()

    # Create exportFileName buffer that will be added to the archive file
    exportFile = "application=%s\n" % os.path.basename(self.appExePath)
    exportFile = exportFile + "brand=%s\n" % strAppBrand
    exportFile = exportFile + "sub_brand=%s\n" % strAppSubBrand
    exportFile = exportFile + "platform=%s\n" % self.appPlatform
    exportFile = exportFile + "version=%s\n" % strAppVersion
    exportFile = exportFile + "installDir=%s\n" % self.appInstDir
    exportFile = exportFile + "smartUpdateVersion=%s\n" % __version__
    exportFile = exportFile + "automatic_start=%s\n" % self.appSvcAutoStart
    exportFile = exportFile + "restart_failure=%s\n" % self.appSvcRestartOnFail


    # zip config directory
    # make sure the output directory is created
    if not os.path.exists(self.outputDir):
      try:
        os.makedirs(self.outputDir)
      except e:
        # Raised error can be because the directory already exists or it could
        # not be created. Since we already checked if it existed, we can assume
        # makedirs failed because it could not create the directory.
        log_error(
          ConfigExporterLogger,
          "Unable to create output directory %s" % self.outputDir
        )
        return False
    
    archivePath = os.path.join(self.outputDir, self.archiveName)
    zip = ZipFile(archivePath, 'w', ZIP_DEFLATED)

    log_info(ConfigExporterLogger,"Creating archive at %s" % archivePath)

    for base, dirs, files in os.walk(self.exportCfgPath):
      for file in files:
        fn = os.path.join(base, file)
        fnZipPath = fn.replace(self.exportCfgPath, '')
        log_debug(
          ConfigExporterLogger,"adding file %s to zip path %s" % 
          (fn, fnZipPath))
        zip.write(fn, fnZipPath)

    log_debug(ConfigExporterLogger, "adding file %s to zip" %
              self.exportFileName)
    zip.writestr(self.exportFileName, exportFile)
    zip.close()

    return True


#-----------------------------------------------------------------------------#
# @method exportCfg : Starts the Smart Update process.
# @param in_strCfgArchivePath: path to the configuration archive to be imported
# @return 0 : export succeeded
# @return 200 : export application failure
#-----------------------------------------------------------------------------#
def exportCfg(in_strExportCfgPath, in_strOutputDir, in_strAppExePath,
              in_strAppInstDir, in_strArchiveName='', 
              in_strAppServiceName=''):

  strExportFilename = "export.txt"

  cfgExporter = ConfigExporter(in_strExportCfgPath, in_strOutputDir,
                               in_strAppExePath, in_strAppInstDir,
                               exportFileName=strExportFilename,
                               archiveName=in_strArchiveName,
                               appServiceName=in_strAppServiceName)

  if cfgExporter.export() is True:
    log_info(fileScopeLogger, "Configuration exported succesfully")
    return 0

  else:
    log_error(fileScopeLogger, "Failed to export configuration")
    return 200


#---------------------------------------------------------------------------#
# Runtest
#---------------------------------------------------------------------------#
if __name__ == "__main__":
  exportCfgDir = "./config"
  outDir = "test-output/"
  appBinPath = ""
  appName = "netborder-gateway"
  appInstDir = "C:\\Program Files\\Netborder\\Express\\Gateway"

  def runtest(in_strExportCfgPath, in_strOutputDir, 
              in_strAppExePath, in_strAppInstDir):
    rc = exportCfg(in_strExportCfgPath, in_strOutputDir, 
                    in_strAppExePath, in_strAppInstDir)
    if rc != 0:
      log_error(fileScopeLogger, "Runtest failed")
    else:
      log_info(fileScopeLogger, "Runtest passed")
    return rc

  
  print runtest(exportCfgDir, outDir, appName, appInstDir)
## !!MIRROR FILE, DO NOT MODIFY!!
