import os
import sys
import subprocess
from subprocess import mswindows
import shutil
from datetime import datetime
import time
import socket

import turbogears
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from paraxip.gateway_gui import extjs
import simplejson
import threading
import netborderPyThread
from netborderPyInfra import PyGlobalConfig as globalConfig
from netborderPyInfra import ParameterInfoDB, ParameterTrie, StringVector

import logging
log = logging.getLogger("pstnconfig.controllers.fxotune")
from netborderPyInfra import Logger
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
fileLogger = Logger("pstnconfig.controllers.fxoTune")

class FxoTuneController(controllers.Controller):

  #CALIBRATION_STATUS
  STATUS_NULL = 0
  STATUS_CALIBRATION_RUNNING = 1
  STATUS_CALIBRATION_COMPLETED = 2
  STATUS_CALIBRATION_CANCELED = 3
  STATUS_CALIBRATION_FAILED = 4

  #FAILED_REASON
  GW_FAILED_TO_START = 6
  GW_FAILED_TO_STOP = 7
  
  def __init__(self, webService, gwStatusController):

    self.gwStatusController = gwStatusController
    
    self.isGatewayInCLI = \
      globalConfig.getParameter('netborder.oamWebServer.isGatewayInCLI')
    
    self.instDir=globalConfig.getParameter("netborder.Installation.Directory")

    strParamDBFile = os.path.join(self.instDir,
        "config/netborder-gateway-paramDefs.xml")
    paramDB = ParameterInfoDB.loadFromXmlFile(strParamDBFile);
    if paramDB is None :
      log.error("failed to load parameter DB from file: " +
                strParamDBFile )

    inputFilePropertiesFiles = os.path.join(self.instDir,
        "config/gw.properties")

    vInputFilePropertiesFiles = StringVector()
    vInputFilePropertiesFiles.append( inputFilePropertiesFiles )

    gatewayParameters = ParameterTrie()
    if gatewayParameters.load(vInputFilePropertiesFiles, paramDB) != True :
        log.error("failed to load properties file(s): " +
                  inputFilePropertiesFiles )

    self.tempConfigFile = "config/temp_fxoTune.conf"
    
    self.mainConfigFile = \
      gatewayParameters.getParameter('netborder.sangoma.fxotune.configPath')
    
    self.fxotune_tool_path = \
      gatewayParameters.getParameter('netborder.sangoma.fxotune.appPath')
    
    self.binDir = \
      globalConfig.getParameter('netborder.Installation.Bin.Directory')
  
    self.strServiceName = \
      globalConfig.getParameter('netborder.oamWebServer.gwServiceName')
      
    self.startGatewayTimeout= \
      globalConfig.getParameter('netborder.oamWebServer.startGwTimeoutMs')
    self.startGatewayTimeout = int(self.startGatewayTimeout)/1000
      
    self.interfacesToCalibrate = [] 
    self.interfacesInCalibration = [] 
    self.interfacesCalibrated = []
    self.interfacesFailedToCalibrate = []
    
    self.__updateCalibrationStatus(self.STATUS_NULL,self.STATUS_NULL)
    
    self.CANCEL_MODE = False
    
    self.webService = webService
    self.fxoTuneProcess = None
    
  
  """
  calibrate each interfaces
  [{wanpipe:x, wanpipe-interface:y}]
  @write the configs inside the mainConfigFile 
  """
  @expose()
  def calibrate(self, interfacesList):

    if self.isGatewayInCLI:
      log.error("FXO Tune don't support Gateway command line mode.")
    

    if not os.path.exists(self.fxotune_tool_path):
      #by default the fxo_tool_path does not have an extension 
      #(to support both linux/windows)
      if not os.path.exists("%s.exe"%self.fxotune_tool_path):
        log.error("netborder.sangoma.fxotune.appPath setted to %s is  " \
                  "not valid. File does not exist. Look at your gateway " \
                  "properties."%(self.fxotune_tool_path))
        return(str(extjs.FormActionLoadResponse(success=False, 
                                           data="error_fxotune_app_not_found")))

    
    if (self.__getCalibrationStatus() == self.STATUS_CALIBRATION_RUNNING):
      return(str(extjs.FormActionLoadResponse(success=False, 
                                     data="error_calibration_already_running")))
    
    interfacesList = simplejson.loads(interfacesList)

    #Load Main config
    self.mainConfig = {}
    if os.path.isfile(self.mainConfigFile):
      self.mainConfig = self.__unSerializeConfigFile(self.mainConfigFile)
    
    #Load Queue
    self.interfacesToCalibrate = interfacesList 
    self.interfacesInCalibration = []
    self.interfacesCalibrated = []
    self.interfacesFailedToCalibrate = []
    
    thread = threading.Thread(target=self.__prepareCalibration)
    thread.start()
    
    return(str(extjs.FormActionLoadResponse(success=True, 
                                           data="success_calibration_started")))
   
  """
  Stop calibration
  MainConfig isn't save. 
  """
  @expose()
  def cancelCalibration(self, _dc=''):
    
    self.__cancelCalibration()
      
    return(str(extjs.FormActionSubmitResponse(success=True))) 
  
  @expose()
  def getCalibrationStatus(self, _dc=''):
    interfacesStatus = []
    for interface in self.interfacesCalibrated:
      tempInterface = interface.copy()
      tempInterface["calibrationStatus"] = "Calibrated"
      interfacesStatus.append(tempInterface)
      
    for interface in self.interfacesInCalibration:
      tempInterface = interface.copy()
      tempInterface["calibrationStatus"] = "In Progress"
      interfacesStatus.append(tempInterface)
      
    for interface in self.interfacesToCalibrate:
      tempInterface = interface.copy()
      tempInterface["calibrationStatus"] = "Waiting"
      interfacesStatus.append(tempInterface)
      
    for interface in self.interfacesFailedToCalibrate:
      tempInterface = interface.copy()
      tempInterface["calibrationStatus"] = "Failed"
      interfacesStatus.append(tempInterface)

    generalStatus=""
    if self.__getCalibrationStatus() == self.STATUS_NULL:
      generalStatus="not_used"
    elif self.__getCalibrationStatus() == self.STATUS_CALIBRATION_RUNNING:
      generalStatus="calibrating"
    elif self.__getCalibrationStatus() == self.STATUS_CALIBRATION_COMPLETED:
      generalStatus="completed"
    elif self.__getCalibrationStatus() == self.STATUS_CALIBRATION_CANCELED:
      generalStatus="canceled"
    elif self.__getCalibrationStatus() == self.STATUS_CALIBRATION_FAILED:
      if self.__getCalibrationFailedReason() == self.GW_FAILED_TO_START:
        generalStatus="gateway_failed_to_start"
      elif self.__getCalibrationFailedReason() == self.GW_FAILED_TO_STOP:
        generalStatus="gateway_failed_to_stop"
      else:
        generalStatus="failed"
      
    return(str(extjs.FormActionLoadResponse(success=True, 
          data={"general":generalStatus,"interfaces":interfacesStatus})))
  
  @expose(template='pstnconfig.templates.fxotune')
  def lightbox_fxotune(self):
    if globalConfig.getParameter("netborder.oamWebServer.offerVirtualBoards"):
      offerVirtualBoards = "true"
    else:
      offerVirtualBoards = "false"

    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    if globalConfig.getParameter("netborder.oamWebServer.enableHardwareDetection"):
      autoDetectHardware = "true"
    else:
      autoDetectHardware = "false"

    startGwTimeout = \
      globalConfig.getParameter("netborder.oamWebServer.startGwTimeoutMs")

    return \
      {
        "footerText":""
        ,"offerVirtualBoards":offerVirtualBoards
        ,"isGatewayInCLI":isGatewayInCLI
        ,"autoDetectHardware":autoDetectHardware
        ,"startGwTimeout":startGwTimeout
      }
  
  def __prepareCalibration(self):
    log.debug("__prepareCalibration")
    self.__updateCalibrationStatus(self.STATUS_CALIBRATION_RUNNING)
    
    gatewayRunningBeforeCalibration = self.__applyPstnConfigToInterfaces()
    
    if self.__getCalibrationStatus() is not self.STATUS_CALIBRATION_FAILED:
      self.__runCalibration()
      
    #Put the gateway in his initial state
    if gatewayRunningBeforeCalibration:
      netborderPyThread.startService(self.strServiceName)
    
    
  def __disableEchoCancellation(self,wanpipeId,interfaceId):
    log.debug( "__disableEchoCancellation" )

    log.debug( "disabling echo canceller on wanpipe%d, "
               "interface%d" % (wanpipeId, interfaceId) )

    try:

      # Call the application to disable the echo cancellation on the
      # specified wanpipe and interface.

      # !+! jpboily : bug 6644 - FXO tune isn't working
      # The tool to disable the echo canceller is not copied anymore so we must
      # use the one provided with the driver, but the executable does not have
      # the same name on linux and windows. Linux's file is wan_ec_client and
      # windows file is wanec_client.
      if os.name == 'posix':

        log.debug( "posix OS" )

        # Here is an example that illustrate how we shall invoke the
        # application: 
        #    wan_ec_client wanpipe1 disable 1 under Linu
        strDisableEchoCmd = 'wan_ec_client'
        strDevice =  "wanpipe%d" % (wanpipeId)
        strInterface = "%d" % interfaceId

      else:

        log.debug( "Windows OS" )

        # Here is an example that illustrate how we shall invoke the
        # application: 
        #    wanec_client wanpipe1_if0 disable 0 
        strDisableEchoCmd = "wanec_client" 
        strDevice =  "wanpipe%d_if%d" % (wanpipeId, interfaceId)
        strInterface = "%d" % interfaceId


      log.debug( "cmd to disable echo canceller: %s %s disable %s" % \
                 (strDisableEchoCmd,strDevice,strInterface) )

      disableEchoProcess = \
          subprocess.Popen(args=[strDisableEchoCmd, 
                                 strDevice,
                                 'disable',
                                 strInterface ],
                           stdout=subprocess.PIPE,
                           stdin=subprocess.PIPE,
                           stderr=subprocess.PIPE)

      output = disableEchoProcess.communicate()
      log.debug( "stdout[%s]. stderr[%s]"%(output[0], output[1]) )

      if  disableEchoProcess.returncode != 0:
        log.error( "failed to disable echo canceller on wanpipe%d, "
                   "interface%d" % (wanpipeId, interfaceId) )
        return False

    except:
      log.error( "failed to disable echo canceller on wanpipe%d, "
                 "interface%d e=%s" % (wanpipeId, interfaceId, sys.exc_info()[0]) )
      return False


    log.debug( "successfully disabled echo canceller on wanpipe%d, "
               "interface%d" % (wanpipeId, interfaceId) )
    return True
      
  def __runCalibration(self):
    log.debug("__runCalibration")
    log.info("Starting FXO interfaces calibration")
    calibrationSucceed = False
    while len(self.interfacesToCalibrate) > 0 and not self.__getCancelMode():
      interface = self.interfacesToCalibrate.pop(0)
      self.interfacesInCalibration.append(interface)
      
      interfaceKey = \
        "s%sc%s"%(interface['wanpipe'],interface['wanpipeInterface'])
    
      if os.path.exists(self.tempConfigFile):
        os.remove(self.tempConfigFile)
       
      if not self.__disableEchoCancellation(interface['wanpipe'], \
                                         interface['wanpipeInterface'] ):
        log.info( "skipping calibration because we failed to disable the " \
                  "echo cancellation on wanpipe %d interface %d" % \
                  (interface['wanpipe'],interface['wanpipeInterface']) )
        continue
        

      log.debug("Starting %s on %s. Calibration will be recorded in %s" \
                %(self.fxotune_tool_path,interfaceKey,self.tempConfigFile))
      self.fxoTuneProcess = subprocess.Popen(args=[self.fxotune_tool_path, 
                                             "-r",
                                             "-i",
                                             "-b",
                                             interfaceKey,
                                             "-e",
                                             interfaceKey,
                                             "-c",
                                             self.tempConfigFile
                                             ],
                                             stdout=subprocess.PIPE,
                                             stdin=subprocess.PIPE,
                                             stderr=subprocess.PIPE)
                                             
      output = self.fxoTuneProcess.communicate()
      fxoLOG = "stdout[%s]. stderr[%s]"%(output[0], output[1])
      
      calibrationSucceed = (self.fxoTuneProcess.returncode == 0)

      self.fxoTuneProcess = None
    
      if calibrationSucceed:
        log.debug("[Succeed] FXO Calibration output generated by %s: %s" \
            %(self.fxotune_tool_path, fxoLOG))
      else:
        log.error("[Failed] FXO Calibration output generated by %s: %s" \
            %(self.fxotune_tool_path, fxoLOG))
      

      if calibrationSucceed and not self.__getCancelMode():
        if os.path.isfile(self.tempConfigFile):
          tempConfig = self.__unSerializeConfigFile(self.tempConfigFile)
          if len(tempConfig.items()) > 0:
            calibrationSucceed = True
            #merge the tempConfig to the mainConfig
            for interface, config in tempConfig.items():
              self.mainConfig[interface] = config
            
          else:
            calibrationSucceed = False
            log.error("Calibration Failed. Configuration file generated " \
            "by %s was empty. Verify if the interface %s (wanpipe: %s, "  \
            "wanpipe-interface: %s) is a valid FXO interface." \
            %(self.fxotune_tool_path,interfaceKey,interface['wanpipe'],
            interface['wanpipeInterface']))

        else:
          calibrationSucceed = False
          log.error("Calibration Failed. Configuration file generated " \
            "by %s was not found"%(self.fxotune_tool_path))
    
      if calibrationSucceed and not self.__getCancelMode():
        log.debug("interfacesInCalibration --> interfacesCalibrated")
        self.interfacesCalibrated.append(self.interfacesInCalibration.pop())
      else:
        self.interfacesFailedToCalibrate.append(
          self.interfacesInCalibration.pop())

    #Serialize Calibration
    if calibrationSucceed and not self.__getCancelMode():
      if os.path.isfile(self.mainConfigFile):
        self.__backupFile(self.mainConfigFile)
      self.__serializeConfigFile(self.mainConfigFile,self.mainConfig)

      self.__updateCalibrationStatus(self.STATUS_CALIBRATION_COMPLETED)

      log.info("Calibration completed with success")
    elif self.__getCancelMode():
      log.info("Calibration was cancelled.")
      self.__calibrationCanceled()
    else:
      log.info("Calibration failed.")
      self.__updateCalibrationStatus(self.STATUS_CALIBRATION_FAILED)
    
  
  """
  Start/Stop the gateway to apply the pstnconfig to the interfaces
  If the gateway is already started, it just stop it.
  @return the innitial status of the gateway (True= Was started, 
  False=Was stopped)
  """
  def __applyPstnConfigToInterfaces(self):
    log.debug("__applyPstnConfigToInterfaces")
    
    gatewayRunningBeforeCalibration = self.__pingGateway()
    
    if(not gatewayRunningBeforeCalibration):
      log.debug("start gateway service")
      netborderPyThread.startService(self.strServiceName)

      startAt = time.time()
      timer = 0
      while(self.__pingGateway() == False and timer < self.startGatewayTimeout):
        log.debug("timer: " + str(timer))
        time.sleep(0.5)
        timer = time.time()-startAt

    if(self.__pingGateway()):
      log.debug("gateway service started")
      netborderPyThread.stopService(self.strServiceName)
      log.debug("stop gateway service")

      #netborderPyThread.isServiceStarted(self.strServiceName)
      while(self.__pingGateway()):
        time.sleep(0.5)
      #wait an additional 1 seconds to be sure the service is 100% stopped.
      time.sleep(1)
      log.debug("gateway service stopped")
      
    else:
      #GATEWAY failed to start
      log.debug("gateway failed to start")
      log.error("The Fxo-Calibration didn't start because the gateway " + \
      "failed to launched. Look at the gataway logs.")
      self.__calibrationFailed()
        
    return gatewayRunningBeforeCalibration
  
  def __pingGateway(self):
    log.debug("__pingGateway")
    return self.gwStatusController.isGatewayReady()
      
  def __updateCalibrationStatus(self,status=0, failedReason=0):
    log.debug("__updateCalibrationStatus")
    self.CALIBRATION_STATUS = status
    self.CALIBRATION_FAILED_REASON = failedReason
    
  def __getCalibrationStatus(self):
    log.debug("__getCalibrationStatus")
    return self.CALIBRATION_STATUS
  
  def __getCalibrationFailedReason(self):
    log.debug("__getCalibrationFailedReason")
    return self.CALIBRATION_FAILED_REASON
  
  def __getCancelMode(self):
    log.debug("__getCancelMode")
    return self.CANCEL_MODE
  
  def __cancelCalibration(self):
    log.debug("__cancelCalibration")
    self.CANCEL_MODE = True
    self.__killCalibration()
    
  def __killCalibration(self):
    log.debug("__killCalibration")
    if self.fxoTuneProcess:
      if mswindows:
        os.system('TASKKILL /PID '+str(self.fxoTuneProcess.pid)+' /F')
      else:
        log.debug("Kill a process on linux (never try before)")
        import signal
        os.kill(self.fxoTuneProcess.pid, signal.SIGTERM)
    
  def __calibrationCanceled(self):
    log.debug("__calibrationCanceled")
    self.interfacesToCalibrate = [] 
    self.interfacesInCalibration = [] 
    self.interfacesCalibrated = []
    self.interfacesFailedToCalibrate = []
    self.__updateCalibrationStatus(self.STATUS_CALIBRATION_CANCELED)
    self.CANCEL_MODE = False
    
    self.interfacesToCalibrate = [] 
    self.interfacesInCalibration = [] 
    self.interfacesCalibrated = []
    self.interfacesFailedToCalibrate = []
    
  def __calibrationFailed(self):
    log.debug("__calibrationFailed")
    self.interfacesFailedToCalibrate.extend(self.interfacesToCalibrate)
    self.interfacesFailedToCalibrate.extend(self.interfacesInCalibration)
    self.interfacesToCalibrate = [] 
    self.interfacesInCalibration = [] 
    
    self.__updateCalibrationStatus(self.STATUS_CALIBRATION_FAILED,
                                   self.GW_FAILED_TO_START)
     
  """
  return a dictionary [interfaces:config]
  @param filePath: path of the config file
  """
  def __unSerializeConfigFile(self, filePath):
    log.debug("__unSerializeConfigFile")
    file = open(filePath, 'r')
    buffer = file.read()
    lines = buffer.splitlines()
    
    config = {}
    for line in lines:
      interface = line.partition("=")
      config[interface[0]] = interface[2]
    file.close()
    return config
    
  """
  serialize a config file
  @param filePath: path of the config file
  @param config: config to serialize (dictionary)
  """
  def __serializeConfigFile(self, filePath, config):
    log.debug("__serializeConfigFile")
    file = open(filePath, 'w')

    for interface, conf in config.items():
      file.write("%s=%s\n"%(interface,conf))
      
      
    fullPath = os.path.abspath(file.name)
    log.debug("fullPath of the config file: " + fullPath)
    
    file.close()
  
  def __backupFile(self, filePath):
    log.debug("__backupFile")
    now = datetime.now()
    fileDate = "%s-%s-%s-%s-%s-%s"%(now.year, now.month, now.day, now.hour, 
                                    now.minute, now.second)
    (filename,extension) = os.path.splitext(self.mainConfigFile)
    backupPath = "%s.%s%s"%(filename,fileDate,extension)
    
    shutil.copyfile(filePath, backupPath)
    
  
